@@ -105,6 +105,11 @@ void checkXYZ(dirCache_t *cache)
 
 int growDirCache(dirCache_t *cache, int slot)
 {
+	if(slot < 0) {
+		fprintf(stderr, "Bad slot %d\n", slot);
+		exit(1);
+	}
+
 	if( cache->nr_entries <= slot) {
 		int i;
 		
@@ -124,6 +129,12 @@ int growDirCache(dirCache_t *cache, int slot)
 dirCache_t *allocDirCache(Stream_t *Stream, int slot)
 {       
 	dirCache_t **dcp;
+
+	if(slot < 0) {
+		fprintf(stderr, "Bad slot %d\n", slot);
+		exit(1);
+	}
+
 	dcp = getDirCacheP(Stream);
 	if(!*dcp) {
 		*dcp = New(dirCache_t);
