@@ -82,18 +82,25 @@ static inline int locate(Fs_t *Stream, int offset, int *slot, int *bit)
 }
 
 static inline int fatReadSector(Fs_t *This, int sector, int slot, 
-				int bit, int dupe)
+				int bit, int dupe, fatBitMask bitmap)
 {
 	int fat_start, ret;
+	int nr_sectors;
 
 	dupe = (dupe + This->primaryFat) % This->num_fat;
 	fat_start = This->fat_start + This->fat_len * dupe;
 	
+	if(bitmap == 0) {
+	    nr_sectors = SECT_PER_ENTRY - bit%SECT_PER_ENTRY;
+	} else {
+	    nr_sectors = 1;
+	}
+
 	/* first, read as much as the buffer can give us */
 	ret = readSector(This,
 					 (char *)(This->FatMap[slot].data+(bit<<This->sectorShift)),
 					 fat_start+sector,
-					 (SECT_PER_ENTRY - bit%SECT_PER_ENTRY));
+					 nr_sectors);
 	if(ret < 0)
 		return 0;
 
@@ -163,13 +170,19 @@ static unsigned char *loadSector(Fs_t *This,
 		ret = -1;
 		for(i=0; i< This->num_fat; i++) {
 			/* read the sector */
-			ret = fatReadSector(This, sector, slot, bit, i);
+			ret = fatReadSector(This, sector, slot, bit, i,
+					    This->FatMap[slot].valid);
 
 			if(ret == 0) {
 				fprintf(stderr,
 					"Error reading fat number %d\n", i);
 				continue;
 			}
+			if(This->FatMap[slot].valid)
+			    /* Set recurs if there have already been
+			     * sectors loaded in this bitmap long
+			     */
+			    recurs = 1;
 			break;
 		}
 
@@ -420,11 +433,13 @@ void fat_write(Fs_t *This)
 		/* initialize info sector */
 		InfoSector_t *infoSector;
 		infoSector = (InfoSector_t *) safe_malloc(This->sector_size);
-		set_dword(infoSector->signature0, INFOSECT_SIGNATURE0);
-		memset(infoSector->filler, sizeof(infoSector->filler),0);
-		set_dword(infoSector->signature, INFOSECT_SIGNATURE);
+		set_dword(infoSector->signature1, INFOSECT_SIGNATURE1);
+		memset(infoSector->filler1, sizeof(infoSector->filler1),0);
+		memset(infoSector->filler2, sizeof(infoSector->filler2),0);
+		set_dword(infoSector->signature2, INFOSECT_SIGNATURE2);
 		set_dword(infoSector->pos, This->last);
 		set_dword(infoSector->count, This->freeSpace);
+		set_dword(infoSector->signature3, 0xaa55);
 		if(forceWriteSector(This, (char *)infoSector, This->infoSectorLoc, 1) !=
 		   This->sector_size)
 			fprintf(stderr,"Trouble writing the info sector\n");
@@ -609,8 +624,8 @@ static int check_media_type(Fs_t *This, struct bootsector *boot,
 		return 0;
 	
 	if((address[0] != boot->descr && boot->descr >= 0xf0 &&
-	    (address[0] != 0xf9 || boot->descr != 0xf0)) ||
-	   address[0] < 0xf0) {
+	    ((address[0] != 0xf9 && address[0] != 0xf7) 
+	     || boot->descr != 0xf0)) || address[0] < 0xf0) {
 		fprintf(stderr,
 			"Bad media types %02x/%02x, probably non-MSDOS disk\n", 
 				address[0],
@@ -646,8 +661,8 @@ static int fat_32_read(Fs_t *This, struct bootsector *boot,
 		infoSector = (InfoSector_t *) safe_malloc(size);
 		if(forceReadSector(This, (char *)infoSector,
 						   This->infoSectorLoc, 1) == This->sector_size &&
-		   _DWORD(infoSector->signature) == INFOSECT_SIGNATURE &&
-		   _DWORD(infoSector->signature0) == INFOSECT_SIGNATURE0) {
+		   _DWORD(infoSector->signature1) == INFOSECT_SIGNATURE1 &&
+		   _DWORD(infoSector->signature2) == INFOSECT_SIGNATURE2) {
 			This->freeSpace = _DWORD(infoSector->count);
 			This->last = _DWORD(infoSector->pos);
 		}
