@@ -13,7 +13,8 @@ typedef struct File_t {
 	struct Fs_t *Fs;	/* Filesystem that this fat file belongs to */
 	Stream_t *Buffer;
 
-	int (*map)(struct File_t *this, off_t where, size_t *len, int mode);
+	int (*map)(struct File_t *this, off_t where, size_t *len, int mode,
+			   mt_off_t *res);
 	size_t FileSize;
 
 	size_t preallocatedSize;
@@ -45,6 +46,11 @@ static File_t *getUnbufferedFile(Stream_t *Stream)
 	return (File_t *) Stream;
 }
 
+Fs_t *getFs(Stream_t *Stream)
+{
+	return getUnbufferedFile(Stream)->Fs;
+}
+
 struct dirCache_t **getDirCacheP(Stream_t *Stream)
 {
 	return &getUnbufferedFile(Stream)->dcp;
@@ -69,7 +75,7 @@ static int recalcPreallocSize(File_t *This)
 	}
 	if(This->preallocatedSize & 0xc0000000) {
 		fprintf(stderr, "Bad preallocated size %x\n", 
-			This->preallocatedSize);
+				(int) This->preallocatedSize);
 	}
 
 	clus_size = Fs->cluster_size * Fs->sector_size;
@@ -79,14 +85,15 @@ static int recalcPreallocSize(File_t *This)
 	neededPrealloc = neededClusters - currentClusters;
 	if(neededPrealloc < 0)
 		neededPrealloc = 0;
-	r = fsPreallocate(Fs, neededPrealloc - This->preallocatedClusters);
+	r = fsPreallocateClusters(Fs, neededPrealloc - This->preallocatedClusters);
 	if(r)
 		return r;
 	This->preallocatedClusters = neededPrealloc;
 	return 0;
 }
 
-static int _loopDetect(int *oldrel, int rel, int *oldabs, int abs)
+static int _loopDetect(unsigned int *oldrel, unsigned int rel, 
+					   unsigned int *oldabs, unsigned int abs)
 {
 	if(*oldrel && rel > *oldrel && abs == *oldabs) {
 		fprintf(stderr, "loop detected! oldrel=%d newrel=%d abs=%d\n",
@@ -102,7 +109,7 @@ static int _loopDetect(int *oldrel, int rel, int *oldabs, int abs)
 }
 
 
-static int loopDetect(File_t *This, int rel, int abs)
+static int loopDetect(File_t *This, unsigned int rel, unsigned int abs)
 {
 	return _loopDetect(&This->loopDetectRel, rel, &This->loopDetectAbs, abs);
 }
@@ -110,7 +117,7 @@ static int loopDetect(File_t *This, int rel, int abs)
 static unsigned int _countBlocks(Fs_t *This, unsigned int block)
 {
 	unsigned int blocks;
-	int rel, oldabs, oldrel;
+	unsigned int rel, oldabs, oldrel;
 
 	blocks = 0;
 	
@@ -134,14 +141,16 @@ unsigned int countBlocks(Stream_t *Dir, unsigned int block)
 	return _countBlocks(This, block);
 }
 
-size_t countBytes(Stream_t *Dir, unsigned int block)
+/* returns number of bytes in a directory.  Represents a file size, and
+ * can hence be not bigger than 2^32
+ */
+static size_t countBytes(Stream_t *Dir, unsigned int block)
 {
 	Stream_t *Stream = GetFs(Dir);
 	DeclareThis(Fs_t);
 
 	return _countBlocks(This, block) * 
-		This->sector_size * 
-		This->cluster_size;
+		This->sector_size * This->cluster_size;
 }
 
 void printFat(Stream_t *Stream)
@@ -149,24 +158,45 @@ void printFat(Stream_t *Stream)
 	File_t *This = getUnbufferedFile(Stream);
 	unsigned long n;
 	int rel;
-	
+	unsigned long begin, end;
+	int first;
+
 	n = This->FirstAbsCluNr;
 	if(!n) {
-		printf("Root directory\n");
+		printf("Root directory or empty file\n");
 		return;
 	}
 
 	rel = 0;
+	first = 1;
+	begin = end = 0;
 	do {
-		printf("<%lu> ",n);
+		if (first || n != end+1) {
+			if (!first) {
+				if (begin != end)
+					printf("-%lu", end);
+				printf("> ");
+			}
+			begin = end = n;
+			printf("<%lu", begin);
+		} else {
+			end++;
+		}
+		first = 0;
 		n = fatDecode(This->Fs, n);
 		rel++;
 		if(loopDetect(This, rel, n) < 0)
 			n = 1;
 	} while (n <= This->Fs->last_fat && n != 1);
+	if(!first) {
+		if (begin != end)
+			printf("-%lu", end);
+		printf(">");
+	}
 }
 
-static int normal_map(File_t *This, off_t where, size_t *len, int mode)
+static int normal_map(File_t *This, off_t where, size_t *len, int mode,
+						   mt_off_t *res)
 {
 	int offset;
 	off_t end;
@@ -178,6 +208,7 @@ static int normal_map(File_t *This, off_t where, size_t *len, int mode)
 	int clus_size;
 	Fs_t *Fs = This->Fs;
 
+	*res = 0;
 	clus_size = Fs->cluster_size * Fs->sector_size;
 	offset = where % clus_size;
 
@@ -248,8 +279,10 @@ static int normal_map(File_t *This, off_t where, size_t *len, int mode)
 			break;
 		CurCluNr++;
 		AbsCluNr = NewCluNr;
-		if(loopDetect(This, CurCluNr, AbsCluNr))
-			return -EIO;
+		if(loopDetect(This, CurCluNr, AbsCluNr)) {
+			errno = EIO;
+			return -2;
+		}
 	}
 
 	maximize(*len, (1 + CurCluNr - RelCluNr) * clus_size - offset);
@@ -265,12 +298,15 @@ static int normal_map(File_t *This, off_t where, size_t *len, int mode)
 		exit(1);
 	}
 
-	return ((This->PreviousAbsCluNr-2) * Fs->cluster_size+Fs->clus_start) *
-			Fs->sector_size + offset;
+	*res = sectorsToBytes((Stream_t*)Fs, 
+						  (This->PreviousAbsCluNr-2) * Fs->cluster_size +
+						  Fs->clus_start) + offset;
+	return 1;
 }
 
 
-static int root_map(File_t *This, off_t where, size_t *len, int mode)
+static int root_map(File_t *This, off_t where, size_t *len, int mode,
+					mt_off_t *res)
 {
 	Fs_t *Fs = This->Fs;
 
@@ -281,42 +317,51 @@ static int root_map(File_t *This, off_t where, size_t *len, int mode)
 	}
 
 	maximize(*len, Fs->dir_len * Fs->sector_size - where);
-	return Fs->dir_start * Fs->sector_size + where;
+        if (*len == 0)
+            return 0;
+	
+	*res = sectorsToBytes((Stream_t*)Fs, Fs->dir_start) + where;
+	return 1;
 }
 	
 
-static int read_file(Stream_t *Stream, char *buf, off_t where, size_t len)
+static int read_file(Stream_t *Stream, char *buf, mt_off_t iwhere, 
+					 size_t len)
 {
 	DeclareThis(File_t);
-	int pos;
+	mt_off_t pos;
+	int err;
+	off_t where = truncBytes32(iwhere);
 
 	Stream_t *Disk = This->Fs->Next;
 	
-	pos = This->map(This, where, &len, MT_READ);
-	if(pos < 0)
-		return pos;
+	err = This->map(This, where, &len, MT_READ, &pos);
+	if(err <= 0)
+		return err;
 	return READS(Disk, buf, pos, len);
 }
 
-static int write_file(Stream_t *Stream, char *buf, off_t where, size_t len)
+static int write_file(Stream_t *Stream, char *buf, mt_off_t iwhere, size_t len)
 {
 	DeclareThis(File_t);
-	int pos;
+	mt_off_t pos;
 	int ret;
 	size_t requestedLen;
 	Stream_t *Disk = This->Fs->Next;
+	off_t where = truncBytes32(iwhere);
+	int err;
 
 	requestedLen = len;
-	pos = This->map(This, where, &len, MT_WRITE);
-	if( pos < 0)
-		return pos;
+	err = This->map(This, where, &len, MT_WRITE, &pos);
+	if( err <= 0)
+		return err;
 	if(batchmode)
 		ret = force_write(Disk, buf, pos, len);
 	else
 		ret = WRITES(Disk, buf, pos, len);
 	if(ret > requestedLen)
 		ret = requestedLen;
-	if ( ret > 0 && where + ret > This->FileSize )
+	if (ret > 0 && where + ret > This->FileSize )
 		This->FileSize = where + ret;
 	recalcPreallocSize(This);
 	return ret;
@@ -329,11 +374,11 @@ static int write_file(Stream_t *Stream, char *buf, off_t where, size_t len)
 
 static int month[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334,
 					  0, 0, 0 };
-static inline long conv_stamp(struct directory *dir)
+static inline time_t conv_stamp(struct directory *dir)
 {
 	struct tm *tmbuf;
 	long tzone, dst;
-	time_t accum;
+	time_t accum, tmp;
 
 	accum = DOS_YEAR(dir) - 1970; /* years past */
 
@@ -377,7 +422,8 @@ static inline long conv_stamp(struct directory *dir)
 	accum += tzone;
 
 	/* correct for Daylight Saving Time */
-	tmbuf = localtime(&accum);
+	tmp = accum;
+	tmbuf = localtime(&tmp);
 	dst = (tmbuf->tm_isdst) ? (-60L * 60L) : 0L;
 	accum += dst;
 	
@@ -385,7 +431,7 @@ static inline long conv_stamp(struct directory *dir)
 }
 
 
-static int get_file_data(Stream_t *Stream, long *date, size_t *size,
+static int get_file_data(Stream_t *Stream, time_t *date, mt_size_t *size,
 			 int *type, int *address)
 {
 	DeclareThis(File_t);
@@ -393,7 +439,7 @@ static int get_file_data(Stream_t *Stream, long *date, size_t *size,
 	if(date)
 		*date = conv_stamp(& This->direntry.dir);
 	if(size)
-		*size = This->FileSize;
+		*size = (mt_size_t) This->FileSize;
 	if(type)
 		*type = This->direntry.dir.attr & ATTR_DIR;
 	if(address)
@@ -406,7 +452,7 @@ static int free_file(Stream_t *Stream)
 {
 	DeclareThis(File_t);
 	Fs_t *Fs = This->Fs;
-	fsPreallocate(Fs, -This->preallocatedClusters);       
+	fsPreallocateClusters(Fs, -This->preallocatedClusters);       
 	FREE(&This->direntry.Dir);
 	freeDirCache(Stream);
 	return hash_remove(filehash, (void *) Stream, This->hint);
@@ -431,10 +477,12 @@ static int flush_file(Stream_t *Stream)
 }
 
 
-static int pre_allocate_file(Stream_t *Stream, size_t size)
+static int pre_allocate_file(Stream_t *Stream, mt_size_t isize)
 {
 	DeclareThis(File_t);
 
+	size_t size = truncBytes32(isize);
+
 	if(size > This->FileSize &&
 	   size > This->preallocatedSize) {
 		This->preallocatedSize = size;
@@ -543,7 +591,7 @@ static Stream_t *_internalFileOpen(Stream_t *Dir, unsigned int first,
 
 	File->Class = &FileClass;
 	File->Fs = This;
-	if(first)
+	if(first || (entry && !IS_DIR(entry)))
 		File->map = normal_map;
 	else
 		File->map = root_map; /* FAT 12/16 root directory */
@@ -570,6 +618,8 @@ Stream_t *OpenRoot(Stream_t *Dir)
 	size_t size;
 	Stream_t *file;
 
+	memset(&entry, 0, sizeof(direntry_t));
+
 	num = fat32RootCluster(Dir);
 
 	/* make the directory entry */
