@@ -28,8 +28,12 @@ static int alloc_ht(T_HashTable *H, int size)
   int i;
 
   for(i=0; sizes[i]; i++)
-    if (sizes[i] > size*2 )
+    if (sizes[i] > size*4 )
       break;
+  if (!sizes[i])
+    for(i=0; sizes[i]; i++)
+      if (sizes[i] > size*2 )
+	break;
   if (!sizes[i])
     for(i=0; sizes[i]; i++)
       if (sizes[i] > size)
@@ -83,23 +87,27 @@ int free_ht(T_HashTable *H, T_HashFunc entry_free)
 }
 
 /* add into hash table without checking for repeats */
-static int _hash_add(T_HashTable *H,T_HashTableEl *E)
+static int _hash_add(T_HashTable *H,T_HashTableEl *E, int *hint)
 {
-  int f2, pos;
+  int f2, pos, ctr;
 
   pos = H->f1(E) % H->size;
   f2 = -1;
+  ctr = 0;
   while(H->entries[pos] != &unallocated &&
 	H->entries[pos] != &deleted){
     if (f2 == -1)
       f2 = H->f2(E) % (H->size - 1);
     pos = (pos+f2+1) % H->size;
+    ctr++;
   }
   if(H->entries[pos] == &unallocated)
      H->fill++; /* only increase fill if the previous element was not yet
 		 * counted, i.e. unallocated */
   H->inuse++;
   H->entries[pos] = E;
+  if(hint)
+	  *hint = pos;
   return 0;
 }
 
@@ -116,25 +124,25 @@ static int rehash(T_HashTable *H)
 
   for(i=0; i < size; i++){
     if(oldentries[i] != &unallocated && oldentries[i] != &deleted)
-      _hash_add(H, oldentries[i]);
+      _hash_add(H, oldentries[i], 0);
   }
   Free(oldentries);
   return 0;
 }
 
-int hash_add(T_HashTable *H, T_HashTableEl *E)
+int hash_add(T_HashTable *H, T_HashTableEl *E, int *hint)
 {
   if (H->fill >= H->max)
     rehash(H);
   if (H->fill == H->size)
     return -1; /*out of memory error */
-  return _hash_add(H,E);
+  return _hash_add(H,E, hint);
 }
 
 
 /* add into hash table without checking for repeats */
-int hash_lookup(T_HashTable *H,T_HashTableEl *E, T_HashTableEl **E2,
-		int *hint)
+static int _hash_lookup(T_HashTable *H,T_HashTableEl *E, T_HashTableEl **E2,
+			int *hint, int isIdentity)
 {
   int f2, pos, upos, ttl;
 
@@ -145,7 +153,8 @@ int hash_lookup(T_HashTable *H,T_HashTableEl *E, T_HashTableEl **E2,
   while(ttl &&
 	H->entries[pos] != &unallocated &&
 	(H->entries[pos] == &deleted ||
-	 H->compar(H->entries[pos], E) != 0)){
+	 ((isIdentity || H->compar(H->entries[pos], E) != 0) &&
+	  (!isIdentity || H->entries[pos] != E)))){
     if (f2 == -1)
       f2 = H->f2(E) % (H->size - 1);
     if (upos == -1 && H->entries[pos] == &deleted)
@@ -166,21 +175,30 @@ int hash_lookup(T_HashTable *H,T_HashTableEl *E, T_HashTableEl **E2,
   return 0;
 }
 
+
+int hash_lookup(T_HashTable *H,T_HashTableEl *E, T_HashTableEl **E2,
+		int *hint)
+{
+	return _hash_lookup(H, E, E2, hint, 0);
+}
+
 /* add into hash table without checking for repeats */
 int hash_remove(T_HashTable *H,T_HashTableEl *E, int hint)
 {
   T_HashTableEl *E2;
 
   if (hint >=0 && hint < H->size &&
-      H->entries[hint] != &unallocated && H->entries[hint] != &deleted &&
-      H->compar(H->entries[hint], E)){
+      H->entries[hint] == E){
     H->inuse--;
     H->entries[hint] = &deleted;
     return 0;
   }
 
-  if(hash_lookup(H, E, &E2, &hint))
-    return -1;
+  if(_hash_lookup(H, E, &E2, &hint, 1)) {
+	  fprintf(stderr, "Removing non-existent entry\n");
+	  exit(1);
+	  return -1;
+  }
   H->inuse--;
   H->entries[hint] = &deleted;
   return 0;
