@@ -9,7 +9,7 @@ typedef int (*T_ComparFunc)(void *, void *);
 
 
 int make_ht(T_HashFunc f1, T_HashFunc f2, T_ComparFunc c, int size, T_HashTable **H);
-int hash_add(T_HashTable *H, T_HashTableEl *E);
+int hash_add(T_HashTable *H, T_HashTableEl *E, int *hint);
 int hash_remove(T_HashTable *H, T_HashTableEl *E, int hint);
 int hash_lookup(T_HashTable *H, T_HashTableEl *E, T_HashTableEl **E2,
 		int *hint);
