@@ -60,13 +60,14 @@ static void usage(void)
 {
 	fprintf(stderr, "Mtools version %s, dated %s\n",
 		mversion, mdate);
-	fprintf(stderr, "Usage: %s [-vscV] drive:\n", progname);
+	fprintf(stderr, "Usage: %s [-vscVn] [-N serial] drive:\n", progname);
 	exit(1);
 }
 
 
 void mlabel(int argc, char **argv, int type)
 {
+    
 	char *newLabel;
 	int verbose, clear, interactive, show, open_mode;
 	direntry_t entry;
@@ -78,7 +79,16 @@ void mlabel(int argc, char **argv, int type)
 	Stream_t *RootDir;
 	int c;
 	int mangled;
-	
+	enum { SER_NONE, SER_RANDOM, SER_SET }  set_serial = SER_NONE;
+	long serial = 0;
+	int need_write_boot = 0;
+	int have_boot = 0;
+	char *eptr = "";
+	struct bootsector boot;
+	Stream_t *Fs=0;
+	int r;
+	struct label_blk_t *labelBlock;
+
 	init_clash_handling(&ch);
 	ch.name_converter = label_name;
 	ch.ignore_entry = -2;
@@ -87,7 +97,7 @@ void mlabel(int argc, char **argv, int type)
 	clear = 0;
 	show = 0;
 
-	while ((c = getopt(argc, argv, "vcs")) != EOF) {
+	while ((c = getopt(argc, argv, "vcsnN:")) != EOF) {
 		switch (c) {
 			case 'v':
 				verbose = 1;
@@ -98,6 +108,21 @@ void mlabel(int argc, char **argv, int type)
 			case 's':
 				show = 1;
 				break;
+			case 'n':
+				set_serial = SER_RANDOM;
+				srandom((long)time (0));
+				serial=random();
+				break;
+			case 'N':
+				set_serial = SER_SET;
+				serial = strtol(optarg, &eptr, 16);
+				if(*eptr) {
+					fprintf(stderr,
+						"%s not a valid serial number\n",
+						optarg);
+					exit(1);
+				}
+				break;
 			default:
 				usage();
 			}
@@ -108,7 +133,8 @@ void mlabel(int argc, char **argv, int type)
 
 	init_mp(&mp);
 	newLabel = argv[optind]+2;
-	interactive = !show && !clear &&!newLabel[0];
+	interactive = !show && !clear &&!newLabel[0] && 
+		(set_serial == SER_NONE);
 	open_mode = O_RDWR;
 	RootDir = open_root_dir(argv[optind][0], open_mode);
 	if(strlen(newLabel) > VBUFSIZE) {
@@ -129,9 +155,13 @@ void mlabel(int argc, char **argv, int type)
 	}
 
 	initializeDirentry(&entry, RootDir);
-	vfat_lookup(&entry, 0, 0, ACCEPT_LABEL | MATCH_ANY,
-		    shortname, longname);
-	
+	r=vfat_lookup(&entry, 0, 0, ACCEPT_LABEL | MATCH_ANY,
+		      shortname, longname);
+	if (r == -2) {
+		FREE(&RootDir);
+		exit(1);
+	}
+
 	if(show || interactive){
 		if(isNotFound(&entry))
 			printf(" Volume has no label\n");
@@ -170,25 +200,46 @@ void mlabel(int argc, char **argv, int type)
 		  0 : 1;
 	}
 
+	have_boot = 0;
+	if( (!show || newLabel[0]) || set_serial != SER_NONE) {
+		Fs = GetFs(RootDir);
+		have_boot = (force_read(Fs,(char *)&boot,0,sizeof(boot)) == 
+			     sizeof(boot));
+	}
+
+	if(_WORD(boot.fatlen)) {
+	    labelBlock = &boot.ext.old.labelBlock;
+	} else {
+	    labelBlock = &boot.ext.fat32.labelBlock;
+	}
+
 	if(!show || newLabel[0]){
-		struct bootsector boot;
-		Stream_t *Fs = GetFs(RootDir);
 
 		if(!newLabel[0])
 			strncpy(shortname, "NO NAME    ",11);
 		else
 			label_name(newLabel, verbose, &mangled, shortname);
 
-		if(force_read(Fs,(char *)&boot,0,sizeof(boot)) == 
-		   sizeof(boot) &&
-		   boot.descr >= 0xf0 &&
-		   boot.ext.old.dos4 == 0x29 &&
-		   _WORD(boot.fatlen)){
-			strncpy(boot.ext.old.label, shortname, 11);
-			force_write(Fs, (char *)&boot, 0, sizeof(boot));
+		if(have_boot && boot.descr >= 0xf0 &&
+		   labelBlock->dos4 == 0x29) {
+			strncpy(labelBlock->label, shortname, 11);
+			need_write_boot = 1;
+
 		}
 	}
 
+	if((set_serial != SER_NONE) & have_boot) {
+		if(have_boot && boot.descr >= 0xf0 &&
+		   labelBlock->dos4 == 0x29) {
+			set_dword(labelBlock->serial, serial);	
+			need_write_boot = 1;
+		}
+	}
+
+	if(need_write_boot) {
+		force_write(Fs, (char *)&boot, 0, sizeof(boot));
+	}
+
 	FREE(&RootDir);
 	exit(result);
 }
