@@ -12,6 +12,8 @@ typedef struct direntry_t {
 			       * root)*/
 	char name[MAX_VNAMELEN+1]; /* name in its parent directory, or 
 				    * NULL if root */
+	int beginSlot; /* begin and end slot, for delete */
+	int endSlot;
 } direntry_t;
 
 #include "stream.h"
@@ -19,7 +21,7 @@ typedef struct direntry_t {
 int vfat_lookup(direntry_t *entry, const char *filename, int length,
 		int flags, char *shortname, char *longname);
 
-struct directory *dir_read(direntry_t *entry);
+struct directory *dir_read(direntry_t *entry, int *error);
 
 void initializeDirentry(direntry_t *entry, struct Stream_t *Dir);
 int isNotFound(direntry_t *entry);
@@ -33,6 +35,6 @@ int labelit(char *dosname,
 	    direntry_t *entry);
 int isSubdirOf(Stream_t *inside, Stream_t *outside);
 char *getPwd(direntry_t *entry);
-void fprintPwd(FILE *f, direntry_t *entry);
+void fprintPwd(FILE *f, direntry_t *entry, int escape);
 int write_vfat(Stream_t *, char *, char *, int, direntry_t *);
 #endif
