@@ -100,18 +100,10 @@ static void cleanup_tty(void)
 { 
 	if(tty && need_tty_reset) {
 		restore_tty (&in_orig);
-		signal (SIGHUP, SIG_IGN);
-		signal (SIGINT, SIG_IGN);
-		signal (SIGTERM, SIG_IGN);
-		signal (SIGALRM, SIG_IGN);
+		setup_signal();
 	}
 }
 
-static void fail(void)
-{
-	exit(1);
-}
-
 static void set_raw_tty(int mode)
 {
 	Terminal in_raw;
@@ -121,14 +113,14 @@ static void set_raw_tty(int mode)
 			/* Determine existing TTY settings */
 			gtty (STDIN, &in_orig);
 			need_tty_reset = 1;
+
+			/* Restore original TTY settings on exit */
 			atexit(cleanup_tty);
 			handlerIsSet = 1;
 		}
 
-		/*------ Restore original TTY settings on exit --------*/
-		signal (SIGHUP, (SIG_CAST) fail);
-		signal (SIGINT, (SIG_CAST) fail);
-		signal (SIGTERM, (SIG_CAST) fail);
+
+		setup_signal();
 		signal (SIGALRM, (SIG_CAST) tty_time_out);
 	
 		/* Change STDIN settings to raw */
